## murahatebu : base.rb

require 'time'
require 'date'
require 'pstore'
require 'cgi'
begin
  require 'cgi/html'
rescue LoadError
end


require 'ttrl/matching_list.rb'
require 'ttrl/text_template.rb'
require 'ttrl/utility.rb'

require 'hatebu.rb'


module MuraHatebu
  VERSION = '1.0.2'

  ## 顼ե
  ERROR_LOG_FILENAME = 'error.log'

  ## ¼ãꥹȥե
  VILLAGE_PEOPLE_FILENAME = 'village_people'

  ## -- ¼Ϥƥַϥǡ¸Υ桼ƥƥؿ
  module Util
    include( TTRL::Utility, Hatebu::Utility )
    self.extend( self )

    def error_log
      File.open( Util.filename_to_path( ERROR_LOG_FILENAME ), "a" ) {|file|
        class << file
          alias :murahatebu_write :write
          def write( str )
            self.murahatebu_write( str )
            $stdout.write( str )
          end
        end
        yield file
      }
    end

    def default_error_log( exception, addition )
      self.error_log {|file|
        file.puts "====== ERROR : #{addition} : #{Time.now.to_standard_string} ====="
        file.puts exception.inspect
        file.puts exception.backtrace
      }
    end

    SORT_OUT_SITE = {}
    [
    'blog.livedoor.jp',
    'd.hatena.ne.jp',
    'twitter.com',
    'ameblo.jp',
    'blogs.yahoo.co.jp',
    ].each {|one| SORT_OUT_SITE[one] = true }
    def get_site_url_from_url( url )
      begin
        uri = URI.parse( url )
        _site = uri.host
        if SORT_OUT_SITE.has_key?( _site )
          _site += ( uri.path.sub( /(\/[^\/]*)\/.*$/, "\\1" ) )
        elsif Regexp.compile( Regexp.escape( "http://b.hatena.ne.jp/articles/" ) ).match( @url )
          _site = 'b.hatena.ne.jp/articles'
        end
        _site
      rescue
        /https?:\/\/([^\/]*)\//.match( url )
        return $1.nil? ? '' : $1
      end
    end

    def make_linked_bookmark_image( url )
      Tag.a( Util.make_bookmark_url( url ) ) {
        Tag.img( Util.make_bookmark_image_url( url ) )
      }
    end

    def date_to_daily_filename( date, type )
      "archives/#{date.to_continuous_string}_#{type}.html"
    end

    def load_village_people_file
      self.loadfile_as_standard_format( Util.filename_to_path( VILLAGE_PEOPLE_FILENAME ) ) {|name|
        yield name
      }
    end
  end

  ## -- ִϢ饹ɤȤǽɲ
  class ::Time
    def to_date
      Date.new( self.year, self.month, self.day )
    end

    def to_standard_string
      self.strftime( '%Y/%m/%d %H:%M:%S' )
    end
  end

  class ::Date
    def to_continuous_string
      self.strftime( '%Y%m%d' )
    end

    def to_standard_string
      self.strftime( '%Y/%m/%d' )
    end

    def to_japanese_string
      self.strftime( '%Yǯ%m%d' )
    end

    def same_month?( other )
      self.year == other.year && self.month == other.month
    end
  end

  ## 饹
  Tag = Object.new
  Tag.extend( CGI::Html4 )
  Tag.element_init()
  Tag.extend( CGI::HtmlExtension )

  ## -- Villager ---------------------------------------------------------
  Villager = Hatebu::User
  class Villager
    alias :initialize_original :initialize
    def initialize( name )
      initialize_original( name )
      @bookmarks = []
    end

    attr_reader( :bookmarks )

    def sort_bookmarks_by_time
      @bookmarks = @bookmarks.sort_by {|bookmark| bookmark.time }
    end
  end

  ## -- Bookmark ---------------------------------------------------------
  Bookmark = Hatebu::Bookmark

  ## -- Entry ------------------------------------------------------------
  Entry = Hatebu::Entry
  class Entry
    TITLE_CONVERT_LIST = TTRL::MatchingList.new
    [
    ['http://anond.hatelabo.jp/', ''],
    ['http://kyoko-np.net/',      'ʹ'],
    ['http://alfalfalfa.com/',    'եե⥶'],
    ['http://digimaga.net/',      'ǥޥ'],
    ].each {|url, site_name|
      TITLE_CONVERT_LIST.add( url ) { site_name }
    }

    def set_title( title )
      @title_original = title
      site_name = TITLE_CONVERT_LIST.match( @target_url )
      @title = site_name ? "#{Util.enc( site_name )} : #{title}" : title
    end
    attr_reader( :title_original )

    alias :get_data_original :get_data
    def get_data
      self.get_data_original
      self.set_title( @title )
    end

    def sort_bookmarks_by_time
      @bookmarks = @bookmarks.sort_by {|bookmark| bookmark.time }
    end
  end

  ## -- DailyData --------------------------------------------------------
  class DailyData
    def initialize( date )
      @date = date
      @cache_filepath = 'cache/' << @date.to_continuous_string << '.dat'
      @data = []
    end

    attr_reader( :date, :cache_filepath, :data )

    def cache_exist?
      File.exists?( @cache_filepath )
    end

    def raise_if_cache_not_exist
      unless self.cache_exist?
        begin
          raise 'cache data is not exist.'
        rescue => e
          Util.default_error_log( e, "date = #{@date.to_standard_string}" )
          raise
        end
      end
    end

    def load_cache
      PStore.new( @cache_filepath ).transaction {|db|
        @data = db['data']
      }
    end

    def save_cache
      PStore.new( @cache_filepath ).transaction {|db|
        db['data'] = @data
      }
    end

    def get_data( villagers )
      @data = []
      villagers.each {|name, villager|
        begin
          villager.get_bookmark {|bookmark|
            next if bookmark.time.to_date > @date
            @data << bookmark if bookmark.time.to_date == @date
            Villager::STOP_GET_BOOKMARK if bookmark.time.to_date < @date
          }
        rescue => e
          Util.default_error_log( e, "villager = #{name} : #{__LINE__}" )
        end
      }
    end
  end

  ## -- CollectedDailyData -----------------------------------------------
  class CollectedDailyData
    def initialize( villagers )
      @villager_hash = {}
      villagers.each {|name, villager|
        @villager_hash[name] = Villager.new( name )
      }
      @entry_hash = Hash.new {|hash, url| hash[url] = Entry.new( url ) }
    end

    attr_reader( :villager_hash, :entry_hash )

    def add_daily_data( daily_data )
      updated_entry = {}
      updated_villager = {}
      daily_data.data.each {|bookmark|
        entry = @entry_hash[bookmark.target_url]
        entry.bookmarks << bookmark
        updated_entry[entry] = true
        if @villager_hash.has_key?( bookmark.user.name )
          villager = @villager_hash[bookmark.user.name]
          villager.bookmarks << bookmark
          updated_villager[villager] = true
        end
      }
      updated_entry.each_key {|entry|
        entry.sort_bookmarks_by_time
        entry.set_title( entry.bookmarks.last.title )
      }
      updated_villager.each_key {|villager| villager.sort_bookmarks_by_time }
    end
  end

  ## -- ̥ƥץ졼
  ## -- DailyTemplate ----------------------------------------------------
  class DailyTemplate < TTRL::TextTemplate
    FILENAME = Util.filename_to_path( 'template/daily_template.html' )
    ENCODE = TTRL::TextTemplate::EncodeProcess::EUC_TO_UTF8
    PARAMS_STRUCT = Struct.new( :date, :type, :version )

    OneRowTemplate = define_new_child( :list, [
      :number,
      :entry_url,
      :title,
      :point,
      :linked_bookmark_image,
      :villager_count]
    )
    class OneRowTemplate
      FaviconTemplate = define_new_child( :favicons, [:villager] )
      CommentTemplate = define_new_child( :comments, [:permalink_url, :villager, :tags, :comment] )
    end

    def add_one_row( number, entry, point )
      self.params.list {|one_row_template|
        one_row_template.params.number = number
        one_row_template.params.entry_url = entry.target_url
        one_row_template.params.title = CGI.escapeHTML( entry.title )
        one_row_template.params.point = point
        one_row_template.params.linked_bookmark_image = Util.make_linked_bookmark_image( entry.target_url )
        one_row_template.params.villager_count = entry.bookmarks.size
        entry.bookmarks.each {|bookmark|
          one_row_template.params.favicons {|favicon_template|
            favicon_template.params.villager = bookmark.user.name
          }
        }
        entry.bookmarks.reverse_each {|bookmark|
          one_row_template.params.comments {|comment_template|
            comment_template.params.permalink_url = bookmark.permalink_url
            comment_template.params.villager = bookmark.user.name
            comment_template.params.tags = bookmark.tags.join( ' ' )
            comment_template.params.comment = CGI.escapeHTML( bookmark.comment.to_s )
          }
        }
      }
    end
  end

  ## -- ֥åꥹȴϢ
  module BlackList
    module FileName
      TWO_CHANNEL = 'url_black_list_2ch'
      ERO         = 'url_black_list_ero'
      SPAM        = 'url_black_list_spam'
      KUSO_MEDIA  = 'url_black_list_kuso_media'
    end

    module Kind
      TWO_CHANNEL = 0
      ERO         = 1
      SPAM        = 2
      KUSO_MEDIA  = 3
    end

    KINDS = [
    Kind::TWO_CHANNEL,
    Kind::ERO,
    Kind::SPAM,
    Kind::KUSO_MEDIA,
    ]

    FILENAMES = [
    FileName::TWO_CHANNEL,
    FileName::ERO,
    FileName::SPAM,
    FileName::KUSO_MEDIA,
    ]

    KIND_TO_FILENAME_TABLE = FILENAMES

    FILENAME_TO_KIND_TABLE = {}
    KIND_TO_FILENAME_TABLE.each_with_index {|filename, index|
      FILENAME_TO_KIND_TABLE[filename] = index
    }
  end
end
