## murahatebu : nocomment.rb

require 'pstore'

require 'ttrl/matching_list.rb'
require 'ttrl/text_template.rb'

require 'base.rb'


module MuraHatebu
  ## -- Ƕιɽ륨ȥϰ
  NOCOMMENT_DISPLAY_RECENT_DAY = 4

  ## -- ǡ¸ե
  NOCOMMENT_DATA_FILENAME = 'nocomment_data.dat'

  ## -- ȰɽȤΥե̾
  NOCOMMENT_SITE_FILENAME = 'nocomment_site'

  ## -- ǶιΥե̾
  NOCOMMENT_RECENT_FILENAME = 'nocomment_recent.html'

  ## -- Υե̾
  NOCOMMENT_ALL_FILENAME = 'nocomment_all.html'


  ## -- NoCommentEntryTemplate -------------------------------------------
  class NoCommentEntryTemplate < TTRL::TextTemplate
    FILENAME = Util.filename_to_path( 'template/nocomment_entry_template.html' )
    ENCODE = TTRL::TextTemplate::EncodeProcess::EUC_TO_UTF8
    PARAMS_STRUCT = Struct.new( :title, :entry_url, :linked_bookmark_image, :version )

    VillagerTemplate = define_new_child( :villagers,
      [:villager, :permalink_url, :tags, :comment]
    )
  end

  ## -- NoCommentAllTemplate -------------------------------------------
  class NoCommentAllTemplate < TTRL::TextTemplate
    FILENAME = Util.filename_to_path( 'template/nocomment_all_template.html' )
    ENCODE = TTRL::TextTemplate::EncodeProcess::EUC_TO_UTF8
    PARAMS_STRUCT = Struct.new( :version )

    OneRowTemplate = define_new_child( :list, [:entry_id, :title, :count, :date] )
  end

  ## -- NoCommentRecentTemplate ----------------------------------------
  class NoCommentRecentTemplate < TTRL::TextTemplate
    FILENAME = Util.filename_to_path( 'template/nocomment_recent_template.html' )
    ENCODE = TTRL::TextTemplate::EncodeProcess::EUC_TO_UTF8
    PARAMS_STRUCT = Struct.new( :days, :version )

    define_child( :list, NoCommentAllTemplate::OneRowTemplate )
  end

  ## -- NoCommentData ----------------------------------------------------
  class NoCommentData
    def initialize( filename = Util.filename_to_path( NOCOMMENT_DATA_FILENAME ) )
      @database = PStore.new( filename )
      @database.transaction {
        @entry_hash = @database.fetch( 'data', {} )
        @recent_url = @database.fetch( 'recent', [] )
        @done_date  = @database.fetch( 'done_date', {} )
      }
    end

    def save
      @database.transaction {
        @database['data'] = @entry_hash
        @database['recent'] = @recent_url
        @database['done_date'] = @done_date
      }
    end

    attr_accessor( :entry_hash, :recent_url, :done_date )


    def add_bookmark( bookmark )
      unless @entry_hash.has_key?( bookmark.target_url )
        tmp = Entry.new( bookmark.target_url )
        tmp.get_data
        tmp.bookmarks.clear
        @entry_hash[bookmark.target_url] = tmp
      end
      @entry_hash[bookmark.target_url].bookmarks << bookmark
    end
  end

  ## -- NoCommentManager -------------------------------------------------
  class NoCommentManager
    def initialize( filename = Util.filename_to_path( NOCOMMENT_DATA_FILENAME ) )
      ## -- Ȱɽȥꥹ
      @nocomment_site_list = TTRL::MatchingList.new( Util.filename_to_path( NOCOMMENT_SITE_FILENAME ) )
      @database = PStore.new( filename )
      @updated_url = {}
    end

    ## -- ᥤ
    def start
      ## -- Υǡμ
      self.load_data

      return if @done_date.has_key?( YESTADAY )

      ## -- åɤ߹
      daily_data = DailyData.new( YESTADAY )
      daily_data.raise_if_cache_not_exist
      daily_data.load_cache

      ## -- ǡɲ
      self.add_daily_data( daily_data )
      @updated_url.each_key {|target_url|
        @entry_hash[target_url].sort_bookmarks_by_time
      }

      ## -- ι줿ȥ
      @recent_url = @recent_url.last( NOCOMMENT_DISPLAY_RECENT_DAY - 1 )
      @recent_url.push( @updated_url )

      ## -- 礷ǡ¸
      self.save_data

      ## -- 줿θ html ι
      self.make_updated_entry_html_file

      ## -- ι
      self.make_all_html_file

      ## -- Ƕιι
      self.make_recent_html_file
    end

    def load_data
      @database.transaction {
        @entry_hash = @database.fetch( 'data', {} )
        @recent_url = @database.fetch( 'recent', [] )
        @done_date  = @database.fetch( 'done_date', {} )
      }
    end

    def save_data
      @database.transaction {
        @database['data'] = @entry_hash
        @database['recent'] = @recent_url
        @database['done_date'] = @done_date
      }
    end

    def add_daily_data( daily_data )
      daily_data.data.select {|bookmark| @nocomment_site_list.match( bookmark.target_url )
      }.each {|bookmark|
        unless @entry_hash.has_key?( bookmark.target_url )
          tmp = Entry.new( bookmark.target_url )
          tmp.get_data
          tmp.bookmarks.clear
          @entry_hash[bookmark.target_url] = tmp
        end
        @entry_hash[bookmark.target_url].bookmarks << bookmark
        @updated_url[bookmark.target_url] = true
      }
      @done_date[daily_data.date] = true
    end

    ## -- HTML Ϸ -----
    ## -- 줿θ html ι
    def make_updated_entry_html_file
      entry_template = NoCommentEntryTemplate.new

      @updated_url.keys.each {|url|
        entry = @entry_hash[url]
        entry_template.reset

        entry_template.params.title = CGI.escapeHTML( entry.title )
        entry_template.params.entry_url = entry.target_url
        entry_template.params.linked_bookmark_image = Util.make_linked_bookmark_image( entry.target_url )
        entry.bookmarks.reverse_each {|bookmark|
          entry_template.params.villagers {|villager_template|
            villager_template.params.villager = bookmark.user.name
            villager_template.params.permalink_url = bookmark.permalink_url
            villager_template.params.tags = bookmark.tags.join( ' ' )
            villager_template.params.comment = CGI.escapeHTML( bookmark.comment.to_s )
          }
        }
        entry_template.params.version = VERSION
        entry_template.write_to_file( Util.filename_to_path( "nocomment/#{entry.entry_id}.html" ) )
      }
    end

    ## -- ι
    def make_all_html_file
      all_template = NoCommentAllTemplate.new

      @entry_hash.values.sort_by {|entry| entry.bookmarks.last.time
      }.reverse_each{|entry|
        all_template.params.list {|one_row_template|
          one_row_template.params.entry_id = entry.entry_id
          one_row_template.params.title = CGI.escapeHTML( entry.title )
          one_row_template.params.count = entry.bookmarks.size
          one_row_template.params.date = entry.bookmarks.last.time.to_date.to_standard_string
        }
      }
      all_template.params.version = VERSION
      all_template.write_to_file( Util.filename_to_path( NOCOMMENT_ALL_FILENAME ) )
    end

    ## -- Ƕιι
    def make_recent_html_file
      recent_template = NoCommentRecentTemplate.new

      @recent_url.collect {|updated| updated.keys
      }.flatten.uniq.collect{|url| @entry_hash[url]
      }.sort_by {|entry| entry.bookmarks.last.time
      }.reverse_each {|entry|
        recent_template.params.list {|one_row_template|
          one_row_template.params.entry_id = entry.entry_id
          one_row_template.params.title = CGI.escapeHTML( entry.title )
          one_row_template.params.count = entry.bookmarks.size
          one_row_template.params.date = entry.bookmarks.last.time.to_date.to_standard_string
        }
      }
      recent_template.params.days = NOCOMMENT_DISPLAY_RECENT_DAY
      recent_template.params.version = VERSION
      recent_template.write_to_file( Util.filename_to_path( NOCOMMENT_RECENT_FILENAME ) )
    end

  end
end
## -- main ---------------------------------------------------------------
if $0 == __FILE__
  MuraHatebu::TODAY = Date.parse( ARGV[0] ) rescue Date.today
  MuraHatebu::YESTADAY = MuraHatebu::TODAY - 1
  MuraHatebu::NoCommentManager.new.start
end
