## tt web analyze : analyzer.rb

require 'pstore'

require 'ttrl/text_template.rb'
require 'ttrl/matching_list.rb'

require 'base.rb'


module TtWebAnalyze
  class AnalyzeTemplate < TTRL::TextTemplate
    FILENAME = Util.filename_to_path( 'analyze_template.html' )
    PARAMS_STRUCT = Struct.new( :date, :day_of_week, :access_count, :unique_count, :version )

    AccessGraphTemplate  = define_new_child( :access_graph, [:height, :top, :value] )
    RefererGraphTemplate = define_new_child( :referer_graph, [:url, :width, :value] )
    AgentGraphTemplate   = define_new_child( :agent_graph, [:agent, :width, :value] )
    HostGraphTemplate    = define_new_child( :host_graph, [:host, :width, :value] )
  end

  ## -- Analyzer ---------------------------------------------------------
  class Analyzer
    EXCLUDE_URL_LIST = TTRL::MatchingList.new
    EXCLUDE_URL_LIST.add( 'http://childs.squares.net' )

    OMIT_URL_LIST = TTRL::MatchingList.new
    [
    'http://www.google.co.jp/url',
    'http://www.google.co.jp/search',
    'http://www.google.com/url',
    'http://www.google.com/search',
    'http://www.google.com.my/url',
    'http://search.yahoo.co.jp/search',
    ].each {|url| OMIT_URL_LIST.add( url ) { url } }

    def initialize( date )
      @date = date
      @cache_filepath = 'cache/' << @date.to_continuous_string << '.dat'
    end

    def cache_exist?
      File.exists?( @cache_filepath )
    end

    def make_html
      database = PStore.new( @cache_filepath )
      database.transaction {
        @log = database.fetch( 'data', [] )
      }

      template = AnalyzeTemplate.new

      template.params.date = @date.to_standard_string
      template.params.day_of_week = @date.strftime( '%a' )
      template.params.access_count = @log.size
      template.params.unique_count = Util.to_count_hash( @log ) {|access|
        access.remote_host.to_s.empty? ? access.remote_address.to_s : access.remote_host
      }.size

      access_by_hour = Array.new( 24, 0 )
      @log.each {|access| access_by_hour[access.time.hour] += 1 }
      access_by_hour.each {|access_count|
        template.params.access_graph {|access_template|
          access_template.params.height = access_count.zero? ? 0 :
          (access_count.to_f * 100.0 / access_by_hour.max.to_f).to_i
          access_template.params.top    = access_template.params.height - 20
          access_template.params.value  = access_count
        }
      }

      Util.to_count_hash( @log ) {|access|
        OMIT_URL_LIST.match( access.referer.to_s ) || access.referer.to_s
      }.to_a.sort_by {|url, count| count
      }.reverse_each{|url, count|
        next if url.to_s.empty?
        next if EXCLUDE_URL_LIST.match( url )
        template.params.referer_graph {|referer_template|
          referer_template.params.url   = url
          referer_template.params.width = count * 5
          referer_template.params.value = count
        }
      }

      Util.to_count_hash( @log ) {|access|
        case access.user_agent
        when /(.+)\/(\d)\..*compatible\; MSIE (\d)\./
          agent = "MSIE/#{$3} of #{$1}/#{$2} compatible"
        when /Mozilla\/(\d)\./
          agent = "Mozilla/#{$1}"
        else
          agent = access.user_agent
        end
        agent
      }.to_a.sort_by {|agent, count| count
      }.reverse_each{|agent, count|
        template.params.agent_graph {|agent_template|
          agent_template.params.agent = agent
          agent_template.params.width = count * 5
          agent_template.params.value = count
        }
      }

      Util.to_count_hash( @log ) {|access|
        access.remote_host.to_s.empty? ? access.remote_address.to_s : access.remote_host
      }.to_a.sort_by {|host, count| count
      }.reverse_each{|host, count|
        template.params.host_graph {|host_template|
          host_template.params.host = host
          host_template.params.width = count * 5
          host_template.params.value = count
        }
      }

      template.params.version = VERSION
      template.make_text
    end

    def write_to_file( path )
      Util.file_puts( path, "w", self.make_html )
    end

    def write_to_date_file
      self.write_to_file( Util.filename_to_path( "archives/#{@date.to_continuous_string}.html" ) )
    end
  end
end
