## tt web analyze : index_maker.rb

require 'ttrl/text_template.rb'

require 'base.rb'


module TtWebAnalyze
  ## -- IndexTemplate ----------------------------------------------------
  class IndexTemplate < TTRL::TextTemplate
    FILENAME = Util.filename_to_path( 'index_template.html' )
    PARAMS_STRUCT = Struct.new( :yestaday, :before_yestaday, :version )

    CalendarTemplate = define_new_child( :calendar, [:year, :month, :days_table] )
    class CalendarTemplate
      def set_data_for_making( date, day_hash )
        @params.year = date.year.to_s
        @params.month = date.month.to_s
        @month_first = Date.new( date.year, date.month, 1 )
        @month_last = (@month_first >> 1) - 1
        @day_hash = day_hash
      end

      def each
        @month_first.wday.times{ yield nil }
        @month_first.step( @month_last ) {|date|
          yield date
        }
        (6 - @month_last.wday).times{ yield nil }
      end

      def each_week
        ary = []
        self.each{|date|
          ary << date
          if ary.size >= 7
            yield ary
            ary = []
          end
        }
      end

      def day_to_html( date )
        return  '' if date.nil?
        tmp = date.day.to_s
        tmp = Tag.a( "./archives/#{date.to_continuous_string}.html" ) { tmp } if @day_hash.has_key?( date )
        tmp
      end

      def make_table
        self.each_week{|week|
          row = ''
          week.collect{|date|
            row << Tag.td { self.day_to_html( date ) }
          }
          @params.days_table << Tag.tr { row } << "\n"
        }
      end
    end
  end

  ## --- IndexMaker ------------------------------------------------------
  module IndexMaker
    def self.make
      today           = Time.now.to_date
      yestaday        = today - 1
      before_yestaday = today - 2

      index_template = IndexTemplate.new
      index_template.params.yestaday = yestaday.to_continuous_string
      index_template.params.before_yestaday = before_yestaday.to_continuous_string

      day_hash = {}
      Dir.glob( Util.filename_to_path( 'archives/*.html' ) ) {|filepath|
        m = /archives\/(20([1-3][0-9])(0[1-9]|1[0-2])(0[1-9]|[12][0-9]|3[01]))\.html/.match( filepath )
        begin
          day_hash[Time.parse( m[1] ).to_date] = true if m
        rescue => e
          Util.file_puts( Util.filename_to_path( 'error.log', 'a', "\n" + e.inspect + "\n" ) )
        end
      }
      month_hash = {}
      day_hash.each_key {|date|
        month_hash[Date.new( date.year, date.month, 1 )] = true
      }
      month_hash.keys.sort.reverse_each {|first_day|
        index_template.params.calendar {|calendar_template|
          calendar_template.set_data_for_making( first_day, day_hash )
          calendar_template.make_table
        }
      }

      index_template.params.version = VERSION
      index_template.write_to_file( Util.filename_to_path( 'index.shtml' ) )
    end
  end
end
