
require 'ttrl/releaser.rb'
require 'ttrl/text_template.rb'
require 'ttrl/utility.rb'

require 'base.rb'


base = File.dirname( File.expand_path( __FILE__ ) )

releaser = TTRL::Releaser.new( 'murahatebu', base )

[
"analyze_villagers.rb",
"base.rb",
"hatebu.rb",
"make_cache.rb",
"make_data_from_cache.rb",
"murahatebu.rb",
"nocomment.rb",
"progress.rb",
"simple-json-parser-builder.rb",
"update.rb",
"ssi_update.sh",

"village_people",
"url_black_list_2ch",
"url_black_list_ero",
"url_black_list_spam",
"url_black_list_kuso_media",
"nocomment_site",
"murahachibu",

"default.css",
"murahatebu.css",
"about.html",
"rss.html",
"blacklist.html",

"history.txt",
"zip_release.rb",
].each {|file| releaser.add_file( file ) }

[
"template",
"tt_web_analyze",
"ttrl",
].each {|dir| releaser.add_dir( dir ) }

releaser.add_pre_copy {|file_utils, to_base|
  [
  "archives",
  "cache",
  "nocomment",
  "rss",
  "site_analyzed",
  "villager_analyzed",
  "favicon",
  "favicon/new",
  "favicon/temp",
  "source",
  ].each {|one| file_utils.mkdir( "#{to_base}/#{one}" ) }
}

[
"favicon/check.rb",
"favicon/get_url.rb",
"favicon/move_to_this.rb",
].each {|file| releaser.add_file( file, "./favicon" ) }

releaser.make_zip( 'source', "-#{MuraHatebu::VERSION}-" )


## -- \[XJ֘A
class SourceTemplate < TTRL::TextTemplate
  FILENAME = TTRL::Utility.filename_to_path( 'template/source_template.html' )
  ENCODE = TTRL::TextTemplate::EncodeProcess::EUC_TO_UTF8
  PARAMS_STRUCT = Struct.new( :version )

  ListTemplate = define_new_child( :list, [:version, :filename] )
end

source_template = SourceTemplate.new
source_template.params.version = MuraHatebu::VERSION

source_array = []
Dir.glob( "./source/murahatebu-*.zip" ) {|filename|
  m = /\.\/source\/([^\-]+\-([^\-]+)\-([^\.]*).*)$/.match( filename )
  if m
    filename = m[1]
    version = m[2]
    date = Date.parse( m[3] )
    source_array << [filename, version, date]
  end
}

source_array.sort_by{|f, v, d| d }
.each{|filename, version, date|
  source_template.params.list {|list_template|
    list_template.params.version = version
    list_template.params.filename = filename
  }
}

source_template.write_to_file( "./source/index.html" )
